﻿using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing.Design;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace PokTiles
{
    [Serializable]
    [Editor(typeof(AnimationEditor), typeof(UITypeEditor))]
    [TypeConverter(typeof(SerializableExpandableObjectConverter))]
    public class AnimatedSprite
    {
        public string Name { get; set; }
        public List<Tile> Frames { get; set; } = new List<Tile>();
        public int Delay { get; set; }

        public int CurrentFrame { get; set; }

        [NonSerialized]
        private Stopwatch sw;

        public AnimatedSprite(string Name)
        {
            this.Name = Name;
        }

        public Tile Update()
        {
            if (sw == null) { sw = new Stopwatch(); sw.Start(); }

            if (sw != null && sw.ElapsedMilliseconds > 200)
            {
                sw.Restart();
                CurrentFrame++;
                if (CurrentFrame >= Frames.Count)
                    CurrentFrame = 0;
            }
            return Frames[CurrentFrame];
        }

        public override string ToString()
        {
            return Name;
        }
    }
}
