﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace PokTiles
{
    public partial class FormAnimationEditor : Form
    {
        private int frameIndex { get; set; } = 0;

        public AnimatedSprite Animation { get; internal set; }

        public FormAnimationEditor()
        {
            InitializeComponent();

            //Double bufferd previw
            typeof(Panel).InvokeMember("DoubleBuffered", BindingFlags.SetProperty
            | BindingFlags.Instance | BindingFlags.NonPublic, null,
            pPreview, new object[] { true });
        }

        private void FormAnimationEditor_Load(object sender, EventArgs e)
        {
            ucTilesMap1.SetSpriteSheet(AppSettings.Singleton().LoadedSpriteSheet);
            BindData();
        }

        private void BindData()
        {
            //Reload data
            tbName.Text = Animation.Name;
            foreach (var t in Animation.Frames)
            {
                lbFrames.Items.Add(t);
            }
        }

        private void btnOk_Click(object sender, EventArgs e)
        {
            UnbindData();
            DialogResult = DialogResult.OK;
            Close();
        }

        private void UnbindData()
        {
            Animation.Name = tbName.Text;
            Animation.Frames = lbFrames.Items.Cast<Tile>().ToList();
        }

        private void ucTilesMap1_TileDoubleClick(object sender, Tile selectedTiles, EventArgs e)
        {
            lbFrames.Items.Add(selectedTiles);
        }

        private void timerAnimation_Tick(object sender, EventArgs e)
        {
            frameIndex++;
            if (frameIndex >= lbFrames.Items.Count)
                frameIndex = 0;
            pPreview.Invalidate();
        }

        private void pPreview_Paint(object sender, PaintEventArgs e)
        {
            Graphics g = e.Graphics;
            g.InterpolationMode = InterpolationMode.NearestNeighbor;
            //Draw
            int W = pPreview.Width;
            int H = pPreview.Height;
            if (frameIndex < lbFrames.Items.Count)
            {
                var tile = lbFrames.Items[frameIndex] as Tile;
                if (tile != null)
                {
                    var img=Utility.DrawTileAltered(tile, AppSettings.Singleton().LoadedSpriteSheet.TransparentColor);
                    g.DrawImage(img, 0, 0, W, H);
                }
            }
        }

        private void FormAnimationEditor_KeyDown(object sender, KeyEventArgs e)
        {
            if(lbFrames.SelectedItem!=null)
            {
                var selTile = lbFrames.SelectedItem as Tile;
                if(selTile!=null)
                {
                    Utility.KeyToFunction(e, selTile);
                }
            }
            Invalidate();
        }
    }
}
