﻿using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace PokTiles
{
    [Serializable]
    public class Layer
    {
        public string Name { get; set; } = "";
        public bool Visible { get; set; } = true;
        public int CameraOffsetX { get; set; }
        public int CameraOffsetY { get; set; }

        public int Columns { get; set; }
        public int Rows { get; set; }

        [JsonProperty]
        private Tile[,] Map { get; set; }

        public Layer(int columns, int rows)
        {
            Columns = columns;
            Rows = rows;
            Map = new Tile[Columns, Rows];
        }

        public List<Tile> GetAllTiles()
        {
            List<Tile> tiles = new List<Tile>();
            foreach (var t in Map)
            {
                tiles.Add(t);
            }
            return tiles;
        }

        internal void SetTile(int column, int row, Tile tile)
        {
            int c = Columns != 0 ? Columns : 1;
            int r = Rows != 0 ? Rows : 1;
            Map[column % c, row % r] = tile;
        }

        public Tile GetTile(int column, int row)
        {
            int c = Columns != 0 ? Columns : 1;
            int r = Rows != 0 ? Rows : 1;
            return Map[column % c, row % r];
        }

        public void Resize(int columns, int rows)
        {
            Columns = columns;
            Rows = rows;
            Tile[,] oldLayer = new Tile[Columns, Rows];
            for (int x = 0; x < Columns; x++)
            {
                for (int y = 0; y < Rows; y++)
                {
                    if (x < Map.GetLength(0) && y < Map.GetLength(1))
                    {
                        oldLayer[x, y] = Map[x, y];
                    }
                    else
                    {
                        oldLayer[x, y] = new Tile();
                    }
                }
            }
            Map = oldLayer;
        }



        public override string ToString()
        {
            return Name + " " + Columns + "x" + Rows;
        }
    }
}
