﻿using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace PokTiles
{
    [Serializable]
    public class SpriteSheet
    {
        public string Name { get; set; } = "New";
        public int SpriteWidth { get; set; } = 8;
        public int SpriteHeight { get; set; } = 8;
        public int Columns { get; set; } = 10;
        public int Rows { get; set; } = 10;
        public int SelectedLayerIndex { get; set; } = 0;
        public Layer[] Layers { get; set; } = new Layer[3];

        public Color TransparentColor { get; set; }

        public SpriteSheet(Image ImageSpriteSheet, int SpriteWidth, int SpriteHeight, bool FirstPixelAsTransparent)
        {
            InitLayersMap();

            Layers[0].Name = "L0";
            Layers[1].Name = "L1";
            Layers[2].Name = "L2";

            Layers[0].Visible = true;
            Layers[1].Visible = true;
            Layers[2].Visible = false;

            this.SpriteWidth = SpriteWidth;
            this.SpriteHeight = SpriteHeight;
            if (ImageSpriteSheet != null) { TilesFromImage(ImageSpriteSheet, FirstPixelAsTransparent); }
        }

        public void InitLayersMap()
        {
            for (int i = 0; i < Layers.Count(); i++)
            {
                Layers[i] = new Layer(Columns, Rows);
            }
            for (int x = 0; x < Columns; x++)
            {
                for (int y = 0; y < Rows; y++)
                {
                    foreach (var l in Layers)
                    {
                        l.SetTile(x, y, new Tile());
                    }
                }
            }
        }

        public void TilesFromImage(Image ImageSpriteSheet, bool FirstPixelAsTransparent)
        {
            if (FirstPixelAsTransparent)
                TransparentColor = ((Bitmap)ImageSpriteSheet).GetPixel(0, 0);

            Columns = ImageSpriteSheet.Width / SpriteWidth;
            Rows = ImageSpriteSheet.Height / SpriteHeight;

            InitLayersMap();
            int Id = 0;

            for (int y = 0; y < Rows; y++)
            {
                for (int x = 0; x < Columns; x++)
                {
                    //Extract tiles
                    Tile tile = new Tile();
                    tile.Id = Id;
                    Bitmap sprite = new Bitmap(SpriteWidth, SpriteHeight);
                    Graphics g = Graphics.FromImage(sprite);
                    g.DrawImage(ImageSpriteSheet,
                        new Rectangle(0, 0,
                                    SpriteWidth,
                                    SpriteHeight),
                        new Rectangle(x * SpriteWidth,
                                        y * SpriteHeight,
                                        SpriteWidth,
                                        SpriteHeight),
                        GraphicsUnit.Pixel);
                    tile.Sprite = sprite;
                    //Compile first layers
                    Layers[0].SetTile(x, y, tile);
                    Id++;
                }
            }
        }

        public void Resize()
        {
            foreach (var l in Layers)
            {
                l.Resize(Columns, Rows);
            }
        }

        public Layer GetSelectedLayer()
        {
            return Layers[SelectedLayerIndex];
        }

        public void SaveJson(string path)
        {
            var json = JsonConvert.SerializeObject(this, Formatting.Indented);
            File.WriteAllText(path, json);
        }

        public static SpriteSheet LoadJson(string path)
        {
            try
            {
                var json = File.ReadAllText(path);
                var ss = JsonConvert.DeserializeObject<SpriteSheet>(json);
                return ss;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(ex);
                return new SpriteSheet(null, 8, 8, false);
            }
        }
    }
}
