﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Runtime.Serialization.Formatters.Binary;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace PokTiles
{
    public static class Utility
    {
        public static T Clamp<T>(this T val, T min, T max) where T : IComparable<T>
        {
            if (val.CompareTo(min) < 0) return min;
            else if (val.CompareTo(max) > 0) return max;
            else return val;
        }

        public static T DeepClone<T>(T obj)
        {
            using (var ms = new MemoryStream())
            {
                var formatter = new BinaryFormatter();
                formatter.Serialize(ms, obj);
                ms.Position = 0;

                return (T)formatter.Deserialize(ms);
            }
        }

        public static string ReverseString(string s)
        {
            char[] charArray = s.ToCharArray();
            Array.Reverse(charArray);
            return new string(charArray);
        }

        public static void KeyToFunction(KeyEventArgs e, Tile tile)
        {
            if (e.KeyCode == Keys.D1)
            {
                tile.FlipH = !tile.FlipH;
            }
            if (e.KeyCode == Keys.D2)
            {
                tile.FlipV = !tile.FlipV;
            }
        }

        public static Bitmap DrawTileAltered(Tile tile, Color transparentColor)
        {
            Bitmap img = (Bitmap)tile.Sprite.Clone();
            if (tile.FlipH)
                img.RotateFlip(RotateFlipType.RotateNoneFlipX);
            if (tile.FlipV)
                img.RotateFlip(RotateFlipType.RotateNoneFlipY);

            img.MakeTransparent(transparentColor);
            return img;
        }

        public static IEnumerable<T> DistinctBy<T, TKey>(this IEnumerable<T> items, Func<T, TKey> property)
        {
            return items.GroupBy(property).Select(x => x.First());
        }

       
    }
}
