﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Drawing.Drawing2D;

namespace PokTiles
{
    public partial class ucTilesMap : UserControl
    {
        public bool EnableDraw { get; set; } = false;
        public bool ShowGrid { get; set; } = true;
        public bool ShowProperties { get; set; } = false;

        public float Zoom { get; set; } = 1.0f;
        public float ViewOffsetX { get; set; } = 0.0f;
        public float ViewOffsetY { get; set; } = 0.0f;

        public Tile PaletteTile { get; set; }
        public Tile SelectedTile { get; private set; }
        public Tile OverTile { get; private set; }

        public Point SelectedTileCoordinate { get; private set; }
        public Point OverTileCoordinate { get; private set; }
        private Point MouseDragBegin { get; set; }

        public delegate void TileSelectedHendler(object sender, Tile selectedTiles, EventArgs e);
        public event TileSelectedHendler TileSelected;
        public event TileSelectedHendler TileDoubleClick;

        private Pen PenGrid = new Pen(Color.DarkGray);
        private Pen PenLayerBorder = new Pen(Color.Red);
        private Pen PenOver = new Pen(Color.Black);
        private Pen PenSelect = new Pen(Color.Orange);

        private SpriteSheet spriteSheet = new SpriteSheet(null, 8, 8, false);

        private List<SpriteSheet> undo = new List<SpriteSheet>();

        public ucTilesMap()
        {
            InitializeComponent();
            DoubleBuffered = true;
            Paint += UcTilesMap_Paint;
            MouseDown += UcTilesMap_MouseDown;
            MouseUp += UcTilesMap_MouseUp;
            MouseClick += UcTilesMap_MouseClick;
            MouseDoubleClick += UcTilesMap_MouseDoubleClick;
            MouseWheel += UcTilesMap_MouseWheel;
            MouseMove += UcTilesMap_MouseMove;
        }


        public void SetTransparentColor(Color transparentColor)
        {
            spriteSheet.TransparentColor = transparentColor;

        }

        public void SetSpriteSheet(SpriteSheet spriteSheet)
        {
            this.spriteSheet = spriteSheet;
            ZoomFit();
        }

        public SpriteSheet GetSpriteSheet()
        {
            return spriteSheet;
        }

        public void SetColumns(int value)
        {
            spriteSheet.Columns = value;
            spriteSheet.Resize();
            ZoomFit();
        }

        public void SetRows(int value)
        {
            spriteSheet.Rows = value;
            spriteSheet.Resize();
            ZoomFit();
        }

        public void SelecteLayerIndex(int index)
        {
            spriteSheet.SelectedLayerIndex = index;
            Invalidate();
        }

        private void ucTilesMap_Load(object sender, EventArgs e)
        {
            ZoomFit();
        }

        /// <summary>
        /// Calculate zoom and offset to fit image
        /// </summary>
        public void ZoomFit()
        {
            //Calculate zoom
            float zoomW = (float)Width / (spriteSheet.Columns * spriteSheet.SpriteWidth);
            float zoomH = (float)Height / (spriteSheet.Rows * spriteSheet.SpriteHeight);
            if (zoomW < zoomH)
            {
                Zoom = zoomW;
                ViewOffsetX = 0;
                ViewOffsetY = ((spriteSheet.Rows * spriteSheet.SpriteHeight * Zoom) - Height) / 2.0f;
            }
            else
            {
                Zoom = zoomH;
                ViewOffsetX = ((spriteSheet.Columns * spriteSheet.SpriteWidth * Zoom) - Width) / 2.0f;
                ViewOffsetY = 0;
            }

            //Calculate offsett to cente



            Invalidate();
        }

        private void UcTilesMap_MouseWheel(object sender, MouseEventArgs e)
        {
            Zoom += e.Delta / 1000.0f;
            if (Zoom < 0.1f)
                Zoom = 0.1f;
            Invalidate();
        }

        private void UcTilesMap_MouseUp(object sender, MouseEventArgs e)
        {

        }

        private void PushUndo()
        {
            //Push Undo
            //undo.Add(Utility.DeepClone(spriteSheet));
        }

        private void PullUndo()
        {
            if (undo.Count > 0)
            {
                //spriteSheet = undo.Last();
                //undo.RemoveAt(undo.Count - 1);
                //Invalidate();
            }
        }

        private void SelectTile(Point location)
        {
            ScreenToCoordinate(location, out int ix, out int iy);
            SelectedTileCoordinate = new Point(ix, iy);
            var selLayer = spriteSheet.GetSelectedLayer();
            SelectedTile = selLayer.GetTile(ix, iy);
        }

        private void OverTileSelect(Point location)
        {
            ScreenToCoordinate(location, out int ix, out int iy);
            OverTileCoordinate = new Point(ix, iy);
            var selLayer = spriteSheet.GetSelectedLayer();
            OverTile = selLayer.GetTile(ix, iy);
        }

        internal Layer GetLayerSelected()
        {
            return spriteSheet.GetSelectedLayer();
        }

        private void PlaceTile(Point MouseLocation)
        {
            PushUndo();
            var selectedLayer = spriteSheet.GetSelectedLayer();
            selectedLayer.GetTile(SelectedTileCoordinate.X, SelectedTileCoordinate.Y).CopyFrom(PaletteTile);
        }

        private void UcTilesMap_MouseDown(object sender, MouseEventArgs e)
        {
            OverTileSelect(e.Location);

            if (MouseButtons == MouseButtons.Left)
            {
                SelectTile(e.Location);
                // Place Tile
                if (EnableDraw)
                {
                    PlaceTile(e.Location);
                }
                else
                {
                    PaletteTile = SelectedTile;
                }
                //Rise event
                TileSelected?.Invoke(this, SelectedTile, new EventArgs());
            }
            //Drag maps
            if (MouseButtons == MouseButtons.Middle)
            {
                MouseDragBegin = e.Location;
            }
            //Use a copy of selected tile as palette
            if (MouseButtons == MouseButtons.Right)
            {
                SelectTile(e.Location);
                PaletteTile = Utility.DeepClone(SelectedTile);
                //Rise event
                TileSelected?.Invoke(this, SelectedTile, new EventArgs());
            }
            Invalidate();
        }

        private void UcTilesMap_MouseMove(object sender, MouseEventArgs e)
        {
            OverTileSelect(e.Location);

            //Draw tiles
            if (MouseButtons == MouseButtons.Left)
            {
                if (EnableDraw && PaletteTile != null)
                {
                    SelectTile(e.Location);
                    PlaceTile(e.Location);
                }
            }

            //Drag maps
            if (MouseButtons == MouseButtons.Middle)
            {
                if (ModifierKeys.HasFlag(Keys.Control))
                {
                    spriteSheet.GetSelectedLayer().CameraOffsetX -= (int)((MouseDragBegin.X - e.Location.X) / Zoom);
                    spriteSheet.GetSelectedLayer().CameraOffsetY -= (int)((MouseDragBegin.Y - e.Location.Y) / Zoom);
                }
                else
                {
                    ViewOffsetX += (MouseDragBegin.X - e.Location.X);
                    ViewOffsetY += (MouseDragBegin.Y - e.Location.Y);
                }
                MouseDragBegin = e.Location;
            }
            Invalidate();
        }

        private void UcTilesMap_MouseClick(object sender, MouseEventArgs e)
        {

        }

        private void UcTilesMap_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            TileDoubleClick?.Invoke(this, SelectedTile, e);
        }

        private void ScreenToCoordinate(Point location, out int ix, out int iy)
        {
            float x = location.X + ViewOffsetX;
            float y = location.Y + ViewOffsetY;
            //Scale by zoom factor
            x /= Zoom;
            y /= Zoom;
            //Find cell
            x /= spriteSheet.SpriteWidth;
            y /= spriteSheet.SpriteHeight;

            ix = (int)x.Clamp(0, spriteSheet.Columns - 1);
            iy = (int)y.Clamp(0, spriteSheet.Rows - 1);
        }

        private void validateProperties()
        {
            Zoom = Zoom > 0.1f ? Zoom : 0.1f;
            spriteSheet.Columns = spriteSheet.Columns > 1 ? spriteSheet.Columns : 1;
            spriteSheet.Rows = spriteSheet.Rows > 1 ? spriteSheet.Rows : 1;
        }

        private void UcTilesMap_Paint(object sender, PaintEventArgs e)
        {
            validateProperties();

            Graphics g = e.Graphics;
            g.InterpolationMode = InterpolationMode.NearestNeighbor;
            g.PixelOffsetMode = PixelOffsetMode.Half;
            g.SmoothingMode = SmoothingMode.None;

            g.TranslateTransform(-ViewOffsetX, -ViewOffsetY);
            g.ScaleTransform(Zoom, Zoom);

            //Draw background
            g.Clear(spriteSheet.TransparentColor);
            g.Clear(Color.Black);

            //Draw multiple layers
            foreach (var l in spriteSheet.Layers)
            {
                if (l.Visible) { DrawLayer(g, l); }
            }

            //Draw Grid
            if (ShowGrid)
            {
                PenGrid.Width = 0.01f * Zoom;
                for (int x = 0; x <= spriteSheet.Columns; x++)
                {
                    g.DrawLine(PenGrid, x * spriteSheet.SpriteWidth, 0, x * spriteSheet.SpriteWidth,
                        spriteSheet.Rows * spriteSheet.SpriteHeight);
                }
                for (int y = 0; y <= spriteSheet.Rows; y++)
                {
                    g.DrawLine(PenGrid, 0, y * spriteSheet.SpriteHeight,
                        spriteSheet.SpriteWidth * spriteSheet.Columns, y * spriteSheet.SpriteHeight);
                }

                //Layer border
                PenLayerBorder.Width = 0.01f * Zoom;
                foreach (var l in spriteSheet.Layers)
                {
                    if (l.Visible)
                        g.DrawRectangle(PenLayerBorder, 0, 0,
                            spriteSheet.SpriteWidth * l.Columns, spriteSheet.SpriteHeight * l.Rows);
                }
            }

            if (EnableDraw)
            {
                //Draw Over tile            
                PenOver.Width = 0.01f * Zoom;
                g.DrawRectangle(PenOver, OverTileCoordinate.X * spriteSheet.SpriteWidth, OverTileCoordinate.Y * spriteSheet.SpriteHeight, spriteSheet.SpriteWidth, spriteSheet.SpriteHeight);

                //Draw tile palette
                DrawTile(g, PaletteTile, OverTileCoordinate.X, OverTileCoordinate.Y);
            }
            //Selected tile border
            PenSelect.Width = 0.01f * Zoom;
            g.DrawRectangle(PenSelect, SelectedTileCoordinate.X * spriteSheet.SpriteWidth, SelectedTileCoordinate.Y * spriteSheet.SpriteHeight, spriteSheet.SpriteWidth, spriteSheet.SpriteHeight);

        }

        private void DrawLayer(Graphics g, Layer l)
        {
            var id = 0;
            Tile tile;
            int xN = 0;
            int yN = 0;
            for (int x = 0; x < spriteSheet.Columns; x++)
            {
                for (int y = 0; y < spriteSheet.Rows; y++)
                {
                    xN = l.Columns != 0 ? x % l.Columns : 0;
                    yN = l.Rows != 0 ? y % l.Rows : 0;
                    //Draw Tiles
                    if (x < spriteSheet.Columns &&
                        y < spriteSheet.Rows &&
                        l.GetTile(xN, yN) != null)
                    {
                        tile = l.GetTile(xN, yN);
                        id = tile.Id;
                        DrawTile(g, tile, l.CameraOffsetX + x, l.CameraOffsetY + y);
                    }
                }
            }
        }

        internal Tile GetUnderCursorTile()
        {
            return OverTile;
        }

        internal Point GetUnderCursorCoordinate()
        {
            return OverTileCoordinate;
        }

        private void DrawTile(Graphics g, Tile tile, int x, int y)
        {
            Point pC = new Point(x * spriteSheet.SpriteWidth, y * spriteSheet.SpriteWidth);
            if (tile != null && tile.Sprite != null)
            {
                if (tile.Animated && tile.Animation != null && tile.Animation.Frames.Count > 0)
                {
                    tile = tile.Animation.Update();
                }

                Bitmap img = Utility.DrawTileAltered(tile, spriteSheet.TransparentColor);
                g.DrawImage(img, pC);
            }
            else
            {
                //g.DrawRectangle(Pens.Red, pC.X + (spriteSheet.SpriteWidth / 2), pC.Y + (spriteSheet.SpriteWidth / 2), 1, 1);
            }

            if (ShowProperties)
            {
                if (tile.Solid)
                {
                    g.DrawRectangle(Pens.Yellow, pC.X, pC.Y, spriteSheet.SpriteWidth, spriteSheet.SpriteHeight);
                }
            }
        }


        private void ucTilesMap_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.G)
            {
                ShowGrid = !ShowGrid;
                Invalidate();
            }

            if (e.KeyCode == Keys.P)
            {
                ShowProperties = !ShowProperties;
                Invalidate();
            }

            if (EnableDraw)
            {
                if (e.KeyCode == Keys.Escape)
                {
                    PaletteTile = null;
                }

                if (ModifierKeys.HasFlag(Keys.Control) && e.KeyCode == Keys.Z)
                {
                    PullUndo();
                }

                Utility.KeyToFunction(e, PaletteTile);

            }
            Invalidate();
        }

        private void ucTilesMap_Resize(object sender, EventArgs e)
        {
            ZoomFit();
        }
    }
}
